function opfiles = calc_mfccs_list(listfile,srcprepend,srcext,dstprepend,dstext,skip)
% calc_mfccs_list(listfile,srcprepend,srcext,dstprepend,dstext,skip)
%   Take listile, a list of input MP3 or WAV files and calculate 
%   MFCC features for each one.
%   input file names each have srcprepend prepended and srcext appended;
%   features are written to .htk files with the same root name but
%   with dstprepend prepended and dstext appended.  First <skip>
%   items are skipped (for resumption of interrupted runs).
%   Return a cell array of the output files written.
% 2007-04-30 dpwe@ee.columbia.edu after coversong calclistftrs

%   Copyright (c) 2006 Columbia University.

if nargin < 2; srcprepend = ''; end
if nargin < 3; srcext = ''; end
if nargin < 4; dstprepend = ''; end
if nargin < 5; dstext = ''; end

if nargin < 6; skip = 0; end

[files,nfiles] = listfileread(listfile);

if nfiles < 1
  error(['No sound file names read from list file "',listfile,'"']);
end

for songn = 1:nfiles
  tline = files{songn};

  if length(srcext) > 0
    if tline(end-length(srcext)+1:end) == srcext
      % chop off srcext already there
      tline = tline(1:end-length(srcext));
    end
  else
    % no srcext specified - must be part of input file name
    % separate name and extension for input file
    [srcpath, srcname, srcext, srcvsn] = fileparts(tline);
    tline = fullfile(srcpath,srcname);
  end
    
  % lines are 
  ifname = fullfile(srcprepend,[tline,srcext]);
  ofname = fullfile(dstprepend,[tline,dstext]);
  
  if songn > skip
  
    disp(['song ',num2str(songn),' ',ifname,' -> ',ofname]);

    [ofdir,nn,ee,vv] = fileparts(ofname);
    % Make sure the parent directory exists
    mymkdir(ofdir)
    
    % wav files or mp3 files
    if srcext == '.mp3'
      [d,sr]=mp3read(ifname,0,1,2); 
    else
      [d,sr]=wavread(ifname);
    end

    % Calculate features
    F = melfcc(d*5.5289, sr, 'lifterexp', 0.6, 'nbands', 19, ...
               'dcttype', 4, 'maxfreq', 8000, 'fbtype', 'fcmel', ...
               'preemph', 0, 'wintime', 0.032, 'hoptime', 0.016, ...
               'numcep', 20, 'dither', 1);

    % Save as HTK
    writehtk(ofname,F',0.010,8192+6);

    % Update progress
    disp([datestr(rem(now,1),'HH:MM:SS'), ' ', ifname,' ncols=',num2str(size(F,2))]);

  end
  
  opfiles{songn} = ofname;
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = mymkdir(dir)
% r = mymkdir(dir)
%   Ensure that dir exists by creating all its parents as needed.
% 2006-08-06 dpwe@ee.columbia.edu

[x,m,i] = fileattrib(dir);
if x == 0
  [pdir,nn,ee,vv] = fileparts(dir);
  mymkdir(pdir);
  disp(['creating ',dir,' ...']);
  mkdir(pdir, nn);
end
