function [p,mahal] = gaussian_prob_invC(x, m, invC, use_log)
% GAUSSIAN_PROB Evaluate a multivariate Gaussian density, given
% inverse covariance
% p = gaussian_prob(X, m, invC, use_log)
%
% p(i) = N(X(:,i), m, C) if use_log = 0 (default)
% p(i) = log N(X(:,i), m, C) if use_log = 1. This prevents underflow.
%
% If X has size dxN, then p has size Nx1, where N = number of examples

if nargin < 4, use_log = 0; end

if length(m)==1 % scalar
  x = x(:)';
end
[d N] = size(x);
%assert(length(m)==d); % slow
m = m(:);
M = m*ones(1,N); % replicate the mean across columns
denom = (2*pi)^(d/2)*sqrt(abs(1/det(invC)));
mahal = sum(((x-M)'*invC).*(x-M)',2);   % Chris Bregler's trick
if any(mahal<0)
  warning('mahal < 0 => C is not psd')
end
if use_log
  p = -0.5*mahal - log(denom);
else
  p = exp(-0.5*mahal) / (denom+eps);
end
