function [acc,confusm,lhood] = test_models(models, testset, nsamp, ...
                                           dims, guard, verb)
% [acc,confusm,lhood] = test_models(models, testset, nsamp, dims, guard, verb)
%     Simply perform testing with pretrained models
% 2007-04-17 dpwe@ee.columbia.edu   part of chromaclassif

% Test on how many samples per file?
if nargin < 3; nsamp = 1000; end
% Using which cepstral dimensions?
if nargin < 4; dims = 1:12; end
if nargin < 5; guard = 20; end
if nargin < 6; verb = 0; end

% where to find the list files
listdir = '../mandelset';

% which kind of features to use
%features = 'mfcc';
features = 'chroma';
disp(['Features = ',features]);

%%%% Configuration
if strcmp(features,'mfcc')
  datapath = '../artists/';
  dataext = '.htk';
  ndims = 20;
elseif strcmp(features,'chroma')
  datapath = '../chromfeats/';
  dataext = '.mat';
  ndims = 12;
else
  error(['Unknown features ',features]);
end

% Reset RNG for consistent results
rand('state', 0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

test_files = listfileread(fullfile(listdir, testset));
test_labs = listfileread(fullfile(listdir, ['labels-',testset]));

ulabs = unique(test_labs);  % hope it matches models
nlabs = length(ulabs);
if nlabs ~= length(models)
  error('number of models doesnt match test set labels')
end

ntest = length(test_files);

% Eval likelihood of each dataset on each model

for file = 1:ntest

  if verb; disp(['testing ',test_files{file},'...']); end
  
  filename = fullfile(datapath,[test_files{file},dataext]);
  d = readdatafile(filename);
  
  % choose random time samples
  ntsamp = nsamp;
  % ri will consist of ntsamp random indices (with replacement) at least 
  % <guard> samples away from end, where guard is larger of 1000 or one
  % quarter of the dataset length
  myguard = min(round(size(d,2)/4),guard); % how far to stay away from ends
  ri = myguard + ceil((size(d,2)-2*myguard)*rand(1,ntsamp));
  
  d = d(dims,ri);
  
  % Evaluate likelihood of this data under every model
  for model = 1:nlabs
    lhood(model,file) = model_match(models(model), d');
%    lhood(model,file) = model_match_chroma(models(model), d);
  end

  % Which model should it have been according to ground truth label?
  gt(file) = find(strcmp(ulabs, test_labs{file}));
  
end

% Which is the most likely model for each test item?
[maxv,indx] = max(lhood);

% Overall error rate
acc = mean(indx==gt);
disp(['Classification accuracy = ', num2str(100*acc),'%']);

% Matrix of which track was classified to which class, and ground truth;
cm = 0*lhood;
gtm = 0*lhood;
for i = 1:ntest
  cm(indx(i),i) = 1;
  gtm(gt(i),i) = 1;
end
% So confusion matrix
confusm = gtm*cm';
% rows are true class, columns are reported (model) class
