% vary_parms.m
%
% Try artist ID modeling with varying parameters
% 2007-04-04 dpwe@ee.columbia.edu

if exist('gmm') == 0
  if fexist('/homes/drspeech/share/lib/matlab/netlab')
    addpath('/homes/drspeech/share/lib/matlab/netlab');
    addpath('/homes/drspeech/share/lib/matlab/KPMtools');
    addpath('/homes/drspeech/share/lib/matlab/KPMstats');
  else
    addpath('/Users/dpwe/matlab/netlab');
    addpath('/Users/dpwe/matlab/KPMtools');
    addpath('/Users/dpwe/matlab/KPMstats');
  end
end

trainset = 'tracks-train.txt';
testset = 'tracks-test.txt';

rslts = [];

style = 1;
doplot = 1;

if style == 0

  % Exhaustive 3D grid search
  
  nsamps = [500 1000 2000 4000];
  ngmms = [8 16 32 64];
  nceps = [16 20];

  for ns = 1:length(nsamps)
    nsamp = nsamps(ns);  

    for ng = 1:length(ngmms)
      ngmm = ngmms(ng);  

      for nc = 1:length(nceps)
	ncep = nceps(nc);
	dims = 1:ncep;
      
	[acc, confusm, lhood] = do_expt(trainset, testset, ngmm, nsamp, dims);
      
	disp([datestr(rem(now,1),'HH:MM:SS'),' nsamp=',num2str(nsamp),' ngmm=',num2str(ngmm),' ncep=',num2str(ncep),' acc=',num2str(acc)]);
	fprintf(1,'%2d ',diag(confusm)); fprintf(1,'\n');
      
	rslts = [rslts,acc];
	
      end
  
    end

  end

end

if style == 1
  
  % Explicit enumeration of every experiment
  % .. which specify 3 line searches
  
  % ngmm nsamp dimmin dimmax (meanacc accsd (filled in later))
  expts = [ 1  250  1  20 .4406 .0218 ; ...
            1  500  1  20 .4785 .0073 ; ...
	    1 1000  1  20 .5080 .0135 ; ...
	    1 2000  1  20 .5044 .0140 ; ...
	    1 4000  1  20 .5129 .0076 ; ...
	    1  250  2  20 .4729 .0124 ; ...
            1  500  2  20 .5124 .0137 ; ...
	    1 1000  2  20 .5317 .0077 ; ...
	    1 2000  2  20 .5428 .0167 ; ...
	    1 4000  2  20 .5435 .0225 ; ...
	    1 1000  1  4  .1878 .0099 ; ...
            1 1000  1  8  .3195 .0122 ; ...
	    1 1000  1  12 .4086 .0166 ; ...
	    1 1000  1  16 .4732 .0127 ; ...
	    1 1000  1  20 .5027 .0118 ; ...
	    1 1000  2  4  .1612 .0160 ; ...
            1 1000  2  8  .3477 .0092 ; ...
	    1 1000  2  12 .4439 .0130 ; ...
	    1 1000  2  16 .5106 .0092 ; ...
	    1 1000  2  20 .5375 .0078 ; ...
	     1 1000  1  20 .5044 .0136 ; ...
             2 1000  1  20 .3608 .0187 ; ...
	     4 1000  1  20 .3928 .0173 ; ...
	     8 1000  1  20 .4299 .0170 ; ...
	    16 1000  1  20 .4630 .0160 ; ...
	    32 1000  1  20 .4838 .0216 ; ...
	    64 1000  1  20 .4780 .0145 ; ...
	     1 1000  2  20 .5373 .0142 ; ...
             2 1000  2  20 .3785 .0195 ; ...
	     4 1000  2  20 .4277 .0153 ; ...
	     8 1000  2  20 .4632 .0157 ; ...
	    16 1000  2  20 .4931 .0142 ; ...
	    32 1000  2  20 .5078 .0149 ; ...
	    64 1000  2  20 .4792 .0190 ];
  
  resultsfile = 'results.out';
  fout = fopen(resultsfile, 'a');
  
  for expt = 1:size(expts,1);
    
    ngmm = expts(expt,1);
    nsamp = expts(expt,2);
    dimmin = expts(expt,3);
    dimmax = expts(expt,4);
    
    dims = dimmin:dimmax;
    
    for rpt = 1:10
      [accs(rpt), confusm, lhood] = do_expt(trainset, testset, ngmm, nsamp, dims);
      correct(rpt,:) = diag(confusm);
    end
    
    macc = mean(accs);
    sdacc = std(accs);
   
    expts(expt,5) = macc;
    expts(expt,6) = sdacc;
    
    rstr = [datestr(rem(now,1),'HH:MM:SS'),' nsamp=',num2str(nsamp),' ngmm=',num2str(ngmm),' ceps=',num2str(dimmin),'..',num2str(dimmax),' macc=',num2str(macc), ' sdacc=',num2str(sdacc)];
    disp(rstr);
    fprintf(fout,'%s\n', rstr);
    cstr = sprintf('%2d ',mean(correct));
    disp(cstr);
    fprintf(fout,'%s\n', cstr);
  
    mcorr(expt,:) = mean(correct);
    
  end

  fclose(fout);

  
  if doplot == 1

    % Plot the three axes of variation
    % 1. performance vs. # training frames
    subplot(231)
    %plot(expts(1:5,2),expts(1:5,5),'.--b',expts(6:10,2),expts(6:10,5),'+-r')
    errorbar(expts(5+[1:5],2),expts(5+[1:5],5),expts(5+[1:5],6),'+-r');
    hold on
    errorbar(expts(1:5,2),expts(1:5,5),expts(1:5,6),'.--b');
    hold off
    grid
    legend('no C0','with C0',4)
    xlabel('# training frames');
    ylabel('classif acc')
    title('Artist ID vs. # training frames (MFCC20, 1 FC Gauss)')
    axis([0 4100 0.35 0.6])
    
    % 2. performance vs. #MFCCs
    subplot(232)
    %plot(expts(10+[1:5],4),expts(10+[1:5],5),'.--b',expts(10+[6:10],4),expts(10+[6:10],5),'+-r')
    errorbar(expts(15+[1:5],4),expts(15+[1:5],5),expts(15+[1:5],6),'+-r');
    hold on
    errorbar(expts(10+[1:5],4),expts(10+[1:5],5),expts(10+[1:5],6),'.--b');
    grid
    hold off
    ylabel('classif acc')
    xlabel('highest MFCC index');
    legend('no C0','with C0',4)
    title('Artist ID vs. highest MFCC coeff (1 FC Gauss, 1000 frames)')
    axis([0 22 0.1 0.6])

    % 3. performance vs. #GMM
    % 1 guass = full covariance (FC)

    subplot(233)
    errorbar(expts(27+[2:7],1),expts(27+[2:7],5),expts(27+[2:7],6),'+-r');
    hold on
    errorbar(expts(20+[2:7],1),expts(20+[2:7],5),expts(20+[2:7],6),'.--b');
    grid
    hold off
    legend('no C0','with C0',4)
    ylabel('classif acc')
    xlabel('# components in GMM')
    title('Artist ID vs. # GMM components (MFCC20, 1000 frames)')
    set(gca,'XTick',[1 10:10:70])
    set(gca,'XTickLabel','FC|10|20|30|40|50|60|70')
    axis([-5 70 .35 .6])
    hold on
    errorbar(expts(20+[1],1),expts(20+[1],5),expts(20+[1],6),'.--b');
    errorbar(expts(27+[1],1),expts(27+[1],5),expts(27+[1],6),'+-r');
    %    print -depsc -adobecset aid-baseline-results.eps
  
  end
    
end


	 
	    
